#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
 * ------------------------------------------------------------------------
 * bmTraces
 * ------------------------------------------------------------------------
 * @license : GNU Affero General Public License Version 3 <http://www.gnu.org/licenses>
 * @author  : Pascal SALAUN 
 * @Website : http://www.bm-stats.org <http://www.bm-stats.org>
 * ------------------------------------------------------------------------
"""

# Native Python Lib/Class 
from Tkinter import * 
import tkSimpleDialog 
import ttk
import threading


# bmTraces Class
from authDialog import *
from mailLogParser import *
from searchDialog import *
from sqlQuery import *
from viewData import *
from waitForTask import *
from Alert import *

class bmTraces:

	control       = 'ko'
	usersDb       = "./db/users.db"
	userOwnDbPath = "/var/lib/bmTraces/"
	
	def __init__(self):
		self.fenetre = Tk()
		self.fenetre.geometry("1100x600+50+50")
		self.fenetre.wm_title("bmTraces : Search in Postfix mail logs ")
		self.fenetre['bg']='#002E6A'
		
		self.mainLogo=PhotoImage(file="./images/bmTracesMain.gif")
		self.backgroundImg = Label(self.fenetre, image=self.mainLogo, bd=0)
		self.backgroundImg.place(x=0, y=0, relwidth=1, relheight=1)
		self.backgroundImg.pack()
				
	def start(self):
		self.initIcons()
		self.initMenu(self.control)
		self.loop()

	def init2Frames(self):
		"""
		Memento : 
		Labelframes don't support scrolling. 
		However, there is a simple solution. 
		First, add a canvas as a child to the labelframe and pack it so that it fills the labelframe. 
		Attach scrollbars to the canvas and add them to the labelframe too. 
		Then embed a frame within the canvas, add your widgets to that inner frame, 
		and then adjust the scrollregion of the canvas to match the size of the frame after you've
		added all the inner labels and entries.
		"""
		
		# Destroy All PanedWindow opened before in self.fenetre
		for instance in self.fenetre.winfo_children():
			if isinstance(instance, PanedWindow):
				instance.destroy()
		
		self.pane = PanedWindow(self.fenetre, orient=HORIZONTAL, sashrelief="sunken")
		self.pane.pack(fill=BOTH, expand=1)
		self.pane.config(bd=2, sashpad=5, bg="white")
		
		"""
		All about the left pannel	
		"""
		
		self.left = left = self.frameL = LabelFrame(self.pane, text="  List of message(s)  ", bg="white")
		self.pane.add(self.left)
		self.pane.paneconfigure(self.left, minsize=400, sticky="nsew")
		
		"""
		All about the right pannel
		vsbR : vertical scrollbar Right
		"""	
		def _configure_frameR(event):
			# update the scrollbars to match the size of the inner frame
			size = (frameR.winfo_reqwidth(), frameR.winfo_reqheight())
			canvasR.config(scrollregion="0 0 %s %s" % size)
			if frameR.winfo_reqwidth() != canvasR.winfo_width():
				# update the canvas's width to fit the inner frame
				canvasR.config(width=frameR.winfo_reqwidth())
		
		self.right = right = LabelFrame(self.pane, text="  Details  ", bg="white")
		self.pane.add(self.right, minsize=650, sticky="nsew")
		
		"""See Memento """
		canvasR = Canvas(right, borderwidth=0)
		self.frameR = frameR = Frame(canvasR, bg="white")
		vsbR = Scrollbar(right, orient="vertical", command=canvasR.yview)
		canvasR.configure(yscrollcommand=vsbR.set, width=650, bg="white", bd=0)       
		vsbR.pack(side="right", fill="y")
		canvasR.pack(side="left", fill="both", expand=True)
		canvasR.create_window((4,4), window=frameR, anchor="nw", tags="frameR")
		
		frameR.bind('<Configure>', _configure_frameR)
		

		
		
	def initIcons(self):
		self.iconLogin=PhotoImage(file="./images/login.gif")
		self.iconLogout=PhotoImage(file="./images/logout.gif")
		self.iconInbox=PhotoImage(file="./images/inbox.gif")
		self.iconOutbox=PhotoImage(file="./images/outbox.gif")
		self.iconText=PhotoImage(file="./images/text.gif")
		self.iconUserList=PhotoImage(file="./images/userList.gif")
		self.iconUserAdd=PhotoImage(file="./images/userAdd.gif")
		self.iconUserDelete=PhotoImage(file="./images/userDelete.gif")
		self.iconUserReset=PhotoImage(file="./images/userReset.gif")
		self.iconSearch=PhotoImage(file="./images/search.gif")
		

	def initMenu(self, control):
		self.initIcons()
		
		menubar = Menu(self.fenetre)

		menu1 = Menu(menubar, tearoff=0)
		if control == "ko":
			menu1.add_command(label="Log in", image=self.iconLogin, compound="left", command=self.authBox)
			menu1.add_separator()
			
		menu1.add_command(label="Quit", image=self.iconLogout, compound="left", command=self.fenetre.destroy)
		menubar.add_cascade(label="File", menu=menu1)

		if control == "ok":
			menu2 = Menu(menubar, tearoff=0)
			menu2.add_command(label="A Sender", image=self.iconInbox, compound="left", command=lambda what='from': self.searchBox(what))
			menu2.add_command(label="A Recipient", image=self.iconOutbox, compound="left", command=lambda what='recip': self.searchBox(what) )
			menu2.add_command(label="A message-id", image=self.iconText, compound="left", command=lambda what='messageid': self.searchBox(what))
			menubar.add_cascade(label="Search", menu=menu2)

			"""
			menu3 = Menu(menubar, tearoff=0)
			menu3.add_command(label="List", image=self.iconUserList, compound="left")
			menu3.add_command(label="Add", image=self.iconUserAdd, compound="left")
			menu3.add_command(label="Delete", image=self.iconUserDelete, compound="left")
			menu3.add_command(label="Reset", image=self.iconUserReset, compound="left")
			menubar.add_cascade(label="Profiles", menu=menu3)
			"""
			
		menu4 = Menu(menubar, tearoff=0)
		menu4.add_command(label="About")
		menubar.add_cascade(label="Help", menu=menu4)

		self.fenetre.config(menu=menubar)

	def loop(self):
		self.fenetre.mainloop()
	
	def authBox(self):
		try: 
			res = authDialog(self.fenetre)
		except TypeError:
			Alert().showinfo('Sorry for you', 'All fields must be filled !!')
			

		try:
			#Sqlite search in users.db
			if res.result[0]: 
				cDb = sqlQuery(self.usersDb)
				t = (res.result[0],)
				password = cDb.selectUser(t)[2]
				cDb.close()
				
				if ( (res.result[1] is not None) and (res.result[1] == password) ) :
					self.userOwnDb = self.userOwnDbPath + res.result[0] + ".db"
					self.initMenu('ok')
					cDb = sqlQuery(self.userOwnDb)
					cDb.initUserDb()
			else:
				Alert().showinfo('Sorry for you', 'All fields must be filled !!')
				
		except TypeError:
			Alert().showinfo('Sorry for you', 'All fields must be filled !!')
			
	def searchBox(self, what):
		self.res = searchDialog(self.fenetre)
		if (self.res.result[0] and self.res.result[1]):
			# Step 0 : Display the waiting page
			self.w = waitForTask()
			t1=threading.Thread(target=self.searchProcess, args=(self.res, what))
			t1.start()
			self.w.start() 
			t1.join()
		else:
			Alert().showinfo('Sorry for you', 'All fields must be filled !!')
			
	def searchProcess(self, res, what):
		self.res = res
		if self.res.result[0]:
			# Step 1 : Parse the file
			p = mailLogParser(self.userOwnDb, res.result[0], what, res.result[1])
			p.getFileContent()
			p.parseContent()
			nbEntry = int(p.getCountEntry()[0])
			p.closeSQL()

			
			if nbEntry > 0:
				# Step 2 : init the 2 panes, left for list of msg, right for details
				self.init2Frames()	
								
				# Step 3 : search in db
				cas = (what, '%'+res.result[1]+'%')
				l = viewData(self.userOwnDb, self.frameL, self.frameR)
				l.getList(cas)
					
				# Step 4 : display list and details
				self.backgroundImg.forget()
				l.displayList()
				self.w.stop() 
				
			else:
				self.w.stop() 
				Alert().showinfo('Sorry for you', 'No entry found !!')
				
				
			
			
		
		
	